package api

import (
	"encoding/json"
	"strconv"

	"git.nemunai.re/filicop/server/models"

	"github.com/julienschmidt/httprouter"
)

func init() {
	router.GET("/missions/", apiHandler(func (_ httprouter.Params, body []byte) (interface{}, error) {
		return filicop.GetMissions()
	}))
	router.DELETE("/missions/", apiHandler(func (_ httprouter.Params, _ []byte) (interface{}, error) {
		return filicop.ClearMissions()
	}))

	router.GET("/missions/:mid", apiHandler(missionHandler(func (mission filicop.Mission, _ []byte) (interface{}, error) {
		return mission, nil
	})))
	router.PUT("/missions/:mid", apiHandler(missionHandler(updateMission)))
	router.DELETE("/missions/:mid", apiHandler(missionHandler(func (mission filicop.Mission, _ []byte) (interface{}, error) {
		return mission.Delete()
	})))

	// Routes inside a requestor
	router.GET("/requestors/:rid/missions/", apiHandler(requestorHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor.GetMissions()
	})))
	router.POST("/requestors/:rid/missions/", apiHandler(requestorHandler(newMission)))
	router.DELETE("/requestors/:rid/mission/", apiHandler(requestorHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor.ClearMissions()
	})))

	router.GET("/requestors/:rid/missions/:mid/", apiHandler(missionInRequestorHandler(func (mission filicop.Mission, _ []byte) (interface{}, error) {
		return mission, nil
	})))
	router.PUT("/requestors/:rid/missions/:mid/", apiHandler(missionInRequestorHandler(updateMission)))
	router.DELETE("/requestors/:rid/missions/:mid/", apiHandler(missionInRequestorHandler(func (mission filicop.Mission, _ []byte) (interface{}, error) {
		return mission.Delete()
	})))

	// Routes inside a requestor inside a fleet
	router.GET("/fleet/:fid/requestors/:rid/missions/", apiHandler(requestorInFleetHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor.GetMissions()
	})))
	router.POST("/fleet/:fid/requestors/:rid/missions/", apiHandler(requestorInFleetHandler(newMission)))
	router.DELETE("/fleet/:fid/requestors/:rid/mission/", apiHandler(requestorInFleetHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor.ClearMissions()
	})))

	router.GET("/fleet/:fid/requestors/:rid/missions/:mid/", apiHandler(missionInRequestorInFleetHandler(func (mission filicop.Mission, _ []byte) (interface{}, error) {
		return mission, nil
	})))
	router.PUT("/fleet/:fid/requestors/:rid/missions/:mid/", apiHandler(missionInRequestorInFleetHandler(updateMission)))
	router.DELETE("/fleet/:fid/requestors/:rid/missions/:mid/", apiHandler(missionInRequestorInFleetHandler(func (mission filicop.Mission, _ []byte) (interface{}, error) {
		return mission.Delete()
	})))
}

func missionHandler(f func(filicop.Mission, []byte) (interface{}, error)) func(httprouter.Params, []byte) (interface{}, error) {
	return func(ps httprouter.Params, body []byte) (interface{}, error) {
		if rid, err := strconv.ParseInt(string(ps.ByName("rid")), 10, 64); err != nil {
			return nil, err
		} else if mission, err := filicop.GetMission(rid); err != nil {
			return nil, err
		} else {
			return f(mission, body)
		}
	}
}

func missionInRequestorHandler(f func(filicop.Mission, []byte) (interface{}, error)) func(httprouter.Params, []byte) (interface{}, error) {
	return func(ps httprouter.Params, body []byte) (interface{}, error) {
		return requestorHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
			if vid, err := strconv.ParseInt(string(ps.ByName("vid")), 10, 64); err != nil {
				return nil, err
			} else if mission, err := requestor.GetMission(vid); err != nil {
				return nil, err
			} else {
				return f(mission, body)
			}
		})(ps, body)
	}
}

func missionInRequestorInFleetHandler(f func(filicop.Mission, []byte) (interface{}, error)) func(httprouter.Params, []byte) (interface{}, error) {
	return func(ps httprouter.Params, body []byte) (interface{}, error) {
		return requestorInFleetHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
			if vid, err := strconv.ParseInt(string(ps.ByName("vid")), 10, 64); err != nil {
				return nil, err
			} else if mission, err := requestor.GetMission(vid); err != nil {
				return nil, err
			} else {
				return f(mission, body)
			}
		})(ps, body)
	}
}

func parseMissionJSON(body []byte, next func (filicop.Mission) (filicop.Mission, error)) (filicop.Mission, error) {
	var um filicop.Mission
	if err := json.Unmarshal(body, &um); err != nil {
		return um, err
	}

	return next(um)
}

func newMission(requestor filicop.Requestor, body []byte) (interface{}, error) {
	return parseMissionJSON(body, requestor.NewMission)
}

func updateMission(mission filicop.Mission, body []byte) (interface{}, error) {
	return parseMissionJSON(body, mission.Update)
}
