package api

import (
	"encoding/json"
	"errors"
	"strconv"

	"git.nemunai.re/filicop/server/models"

	"github.com/julienschmidt/httprouter"
)

func init() {
	router.GET("/requestors/", apiHandler(func (_ httprouter.Params, body []byte) (interface{}, error) {
		return filicop.GetRequestors()
	}))
	router.DELETE("/requestors/", apiHandler(func (_ httprouter.Params, _ []byte) (interface{}, error) {
		return filicop.ClearRequestors()
	}))

	router.GET("/requestors/:rid", apiHandler(requestorHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor, nil
	})))
	router.PUT("/requestors/:rid", apiHandler(requestorHandler(updateRequestor)))
	router.DELETE("/requestors/:rid", apiHandler(requestorHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor.Delete()
	})))

	// Routes inside a fleet
	router.GET("/fleets/:fid/requestors/", apiHandler(fleetHandler(func (fleet filicop.Fleet, _ []byte) (interface{}, error) {
		return fleet.GetRequestors()
	})))
	router.POST("/fleets/:fid/requestors/", apiHandler(fleetHandler(newRequestor)))
	router.DELETE("/fleets/:fid/requestor/", apiHandler(fleetHandler(func (fleet filicop.Fleet, _ []byte) (interface{}, error) {
		return fleet.ClearRequestors()
	})))

	router.GET("/fleets/:fid/requestors/:rid/", apiHandler(requestorInFleetHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor, nil
	})))
	router.PUT("/fleets/:fid/requestors/:rid/", apiHandler(requestorInFleetHandler(updateRequestor)))
	router.DELETE("/fleets/:fid/requestors/:rid/", apiHandler(requestorInFleetHandler(func (requestor filicop.Requestor, _ []byte) (interface{}, error) {
		return requestor.Delete()
	})))
}

func requestorHandler(f func(filicop.Requestor, []byte) (interface{}, error)) func(httprouter.Params, []byte) (interface{}, error) {
	return func(ps httprouter.Params, body []byte) (interface{}, error) {
		if rid, err := strconv.ParseInt(string(ps.ByName("rid")), 10, 64); err != nil {
			return nil, err
		} else if requestor, err := filicop.GetRequestor(rid); err != nil {
			return nil, err
		} else {
			return f(requestor, body)
		}
	}
}

func requestorInFleetHandler(f func(filicop.Requestor, []byte) (interface{}, error)) func(httprouter.Params, []byte) (interface{}, error) {
	return func(ps httprouter.Params, body []byte) (interface{}, error) {
		return fleetHandler(func (fleet filicop.Fleet, _ []byte) (interface{}, error) {
			if vid, err := strconv.ParseInt(string(ps.ByName("vid")), 10, 64); err != nil {
				return nil, err
			} else if requestor, err := fleet.GetRequestor(vid); err != nil {
				return nil, err
			} else {
				return f(requestor, body)
			}
		})(ps, body)
	}
}

func parseRequestorJSON(body []byte, next func (filicop.Requestor) (filicop.Requestor, error)) (filicop.Requestor, error) {
	var ur filicop.Requestor
	if err := json.Unmarshal(body, &ur); err != nil {
		return ur, err
	}

	if len(ur.Name) == 0 {
		return ur, errors.New("Requestor's name cannot be empty")
	}

	return next(ur)
}

func newRequestor(fleet filicop.Fleet, body []byte) (interface{}, error) {
	return parseRequestorJSON(body, fleet.NewRequestor)
}

func updateRequestor(requestor filicop.Requestor, body []byte) (interface{}, error) {
	return parseRequestorJSON(body, requestor.Update)
}
