package api

import (
	"encoding/json"
	"errors"
	"strconv"

	"git.nemunai.re/filicop/server/models"

	"github.com/julienschmidt/httprouter"
)

func init() {
	router.GET("/vehicles/", apiHandler(func (_ httprouter.Params, body []byte) (interface{}, error) {
		return filicop.GetVehicles()
	}))
	router.DELETE("/vehicles/", apiHandler(func (_ httprouter.Params, _ []byte) (interface{}, error) {
		return filicop.ClearVehicles()
	}))

	router.GET("/vehicles/:vid/", apiHandler(vehicleHandler(func (vehicle filicop.Vehicle, _ []byte) (interface{}, error) {
		return vehicle, nil
	})))
	router.PUT("/vehicles/:vid/", apiHandler(vehicleHandler(updateVehicle)))
	router.DELETE("/vehicles/:vid/", apiHandler(vehicleHandler(func (vehicle filicop.Vehicle, _ []byte) (interface{}, error) {
		return vehicle.Delete()
	})))

	// Routes inside a fleet
	router.GET("/fleets/:fid/vehicles/", apiHandler(fleetHandler(func (fleet filicop.Fleet, _ []byte) (interface{}, error) {
		return fleet.GetVehicles()
	})))
	router.POST("/fleets/:fid/vehicles/", apiHandler(fleetHandler(newVehicle)))
	router.DELETE("/fleets/:fid/vehicle/", apiHandler(fleetHandler(func (fleet filicop.Fleet, _ []byte) (interface{}, error) {
		return fleet.ClearVehicles()
	})))

	router.GET("/fleets/:fid/vehicles/:vid/", apiHandler(vehicleInFleetHandler(func (vehicle filicop.Vehicle, _ []byte) (interface{}, error) {
		return vehicle, nil
	})))
	router.PUT("/fleets/:fid/vehicles/:vid/", apiHandler(vehicleInFleetHandler(updateVehicle)))
	router.DELETE("/fleets/:fid/vehicles/:vid/", apiHandler(vehicleInFleetHandler(func (vehicle filicop.Vehicle, _ []byte) (interface{}, error) {
		return vehicle.Delete()
	})))
}

func vehicleHandler(f func(filicop.Vehicle, []byte) (interface{}, error)) func(httprouter.Params, []byte) (interface{}, error) {
	return func(ps httprouter.Params, body []byte) (interface{}, error) {
		if vid, err := strconv.ParseInt(string(ps.ByName("vid")), 10, 64); err != nil {
			return nil, err
		} else if vehicle, err := filicop.GetVehicle(vid); err != nil {
			return nil, err
		} else {
			return f(vehicle, body)
		}
	}
}

func vehicleInFleetHandler(f func(filicop.Vehicle, []byte) (interface{}, error)) func(httprouter.Params, []byte) (interface{}, error) {
	return func(ps httprouter.Params, body []byte) (interface{}, error) {
		return fleetHandler(func (fleet filicop.Fleet, _ []byte) (interface{}, error) {
			if vid, err := strconv.ParseInt(string(ps.ByName("vid")), 10, 64); err != nil {
				return nil, err
			} else if vehicle, err := fleet.GetVehicle(vid); err != nil {
				return nil, err
			} else {
				return f(vehicle, body)
			}
		})(ps, body)
	}
}

func parseVehicleJSON(body []byte, next func (filicop.Vehicle) (filicop.Vehicle, error)) (filicop.Vehicle, error) {
	var uv filicop.Vehicle
	if err := json.Unmarshal(body, &uv); err != nil {
		return uv, err
	}

	if len(uv.Label) == 0 {
		return uv, errors.New("Vehicle's label cannot be empty")
	}

	return next(uv)
}

func newVehicle(fleet filicop.Fleet, body []byte) (interface{}, error) {
	return parseVehicleJSON(body, fleet.NewVehicle)
}

func updateVehicle(vehicle filicop.Vehicle, body []byte) (interface{}, error) {
	return parseVehicleJSON(body, vehicle.Update)
}
